function argout = groupwise_reg(I, T0, MOT_METRIC, MOT_WEIGHTS, usegpu, roi, Spacing)
%GROUPWISE_REG Summary of this function goes here
%   I  : 2D+t secquence ROWS x COLS x FRAMES
%   T0 : Initial grid of control points. Use empty matrix for null initial deformation.
%   REG_ROI : Registration region: 'wholebox' -> square box containing the whole image
%                                  'circular' -> circular region centered at ROI_rad (2-element vector) and of radius ROI_rad
%   MOT_METRIC : 'VI' -> Variance of the Intensity (Sum of Squared Differences).
%   MOT_WEIGHTS : weights of the spatio-temporal regularization term. 4-element vector:
%                              1st element: first order spatial regularization
%                              2nd element: second order spatial regularization
%                              3rd element: first order temporal regularization
%                              4th element: second order temporal regularization
%
%
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.



%% SETUP REGISTRATION
% GROUPWISE REGISTRATION SETUP
if nargin < 7
    Spacing = [2 2]; % b-spline control points spacing in x and y direction
end

s = size(I);
ROW = s(1);
COL = s(2);
N   = s(3);

% Original coordinates grid
[x(:,:,1), x(:,:,2)] = meshgrid(1:COL,1:ROW);
if usegpu
    x = gpuArray(x);
end
dataType = class(x);
% Registration options 
E = 3;           % b-spline order

% Set ROI
switch roi.type
    case 'wholebox'
        Nh = Spacing(1)*(E+1);
        Nv = Spacing(2)*(E+1);
        X = ones([ROW COL],dataType);
        X(1:Nv,:) = 0; X(end-Nv+1:end,:) = 0;
        X(:,1:Nh) = 0; X(:,end-Nh+1:end) = 0;
        Xarea = sum(X(:));
        box = [Nv+1 ROW-Nv; Nh+1 COL-Nh ];
    case 'circular'
        [X,Xarea,box] = circROI_bs([ROW COL],roi.cen,roi.rad);
end


[ts, T] = bspline_init(x,box,Spacing,E,N);
if exist('T0','var') && ~isempty(T0)
    T = T0;
end

% CG optimization parameters for registration
parametros.nmax = 100;           % Max iterations
parametros.pxTh = 0.25;          % Displacement threshold (in pixels)
parametros.et   = 0.001;         % Portion of moving pixels
parametros.eh   = 0.001;         % Prctg

parametros.le   = MOT_WEIGHTS;   % Regularization coefs
parametros.t0   = 1;             % Initial step
parametros.interpolation = 'linear';

if isempty(MOT_METRIC)
    MOT_METRIC = VI;
end

switch MOT_METRIC
    case 'VI'
        parametros.reg_metric    = @VI_metric;
        parametros.reg_gradient  = @VI_gradient;
    case 'VIMA'
        parametros.reg_metric    = @(IT,roi,Nroi)VIMA_metric(IT,roi,Nroi,MOT_ARG);
        parametros.reg_gradient  = @(IT)VIMA_gradient(IT,MOT_ARG);
    otherwise
        error('mot_metric not implemented\n');
end

% Weights matrix


Wn = ones(size(T),dataType)/Xarea;

if any(isnan(I(:)))
    warning('NaNs in input image');
end

Imax = max(abs(I(:)))/700;


%% Do it
fprintf(1,' registration\n');
tic;
[ ~, ~, T_ite, IT] = optimizadorcg_javi( x,ts,T,abs(I)/Imax,X,parametros,Wn);
exectime = toc;
fprintf(1,' registration finished in %d seconds\n',exectime);

argout = struct();
argout.xn = bspline_transform(x,ts,T_ite);
argout.x = x;
argout.ts = ts;
argout.T_ite = T_ite;
argout.exectime = exectime;
argout.IT = IT*Imax;

end

